package cz.cvut.fel.pr1.data;

import cz.cvut.fel.pr1.cars.model.Car;

/**
 * The CarLinkedList class represents a simplified implementation of a linked list of cars.
 * This implementation can be extended with additional operations, e.g., accessing first/last item and removing elements.
 * @see java.util.LinkedList for list of possible features
 * @author balikm1
 */
public class CarLinkedList {
    private int size;
    private CarListNode first;
    
    //<editor-fold defaultstate="collapsed" desc="initialization">
    /**
     * Constructs a newly allocated CarLinkedList object that represents a linked list of cars
     */
    public CarLinkedList() {
        size = 0;
        first = null;
    }
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="public methods">
    /**
     * Appends the specified element to the end of this list.
     * @param car element to be appended to this list
     */
    public void add(Car car) {
        if (first == null) {
            first = new CarListNode(car, null);
        } else {
            CarListNode node = first;
            while (node.getNext() != null) {
                node = node.getNext();
            }
            node.setNext(new CarListNode(car, null));
        }
        size++;
    }
    
    /**
     * Returns the number of elements in this list.
     * @return the number of elements in this list
     */
    public int size() {
        return size;
    }
    
    /**
     * Returns an array containing all of the elements in this list in proper sequence (from first to last element)
     * @return an array containing all the elements in this linked list
     */
    public Car[] toArray() {
        Car[] array = new Car[size];
        int index = 0;
        if (first != null) {
            CarListNode node = first;
            do {
                array[index++] = node.getCar();
                node = node.getNext();
            } while (node != null);
        }
        return array;
    }
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="object overrides">
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        if (first != null) {
            CarListNode node = first;
            do {
                sb.append(node.getCar().toString()).append("\n");
                node = node.getNext();
            } while (node != null);
        }
        return sb.toString();
    }
    //</editor-fold>
}
