package cz.cvut.fel.pr1.data;

import cz.cvut.fel.pr1.cars.model.Car;

/**
 * The RaceResult class represents a key-value pair of Car and its race time result represented by TimeSpan
 * @author balikm1
 */
public class RaceResult implements Comparable<RaceResult> {
    private final Car key;
    private final TimeSpan value;
    
    //<editor-fold defaultstate="collapsed" desc="initialization">
    /**
     * Constructs a newly allocated RaceResult object that represents the key-value pair of Car and its race time result
     * @param key instance of a car
     * @param value car's race time result represented by TimeSpan
     */
    public RaceResult(Car key, TimeSpan value) {
        this.key = key;
        this.value = value;
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="properties">
    /**
     * Get car instance that identifies the pair
     * @return car instance that identifies the pair
     */
    public Car getKey() {
        return key;
    }

    /**
     * Get race time result of the car
     * @return race time result of the car
     */
    public TimeSpan getValue() {
        return value;
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="object overrides">
    @Override
    public int compareTo(RaceResult o) {
        if (value.getTotalSeconds() < o.getValue().getTotalSeconds())
            return -1;
        else if (value.getTotalSeconds() > o.getValue().getTotalSeconds())
            return 1;
        else return 0;
    }
    //</editor-fold>
}
