package cz.cvut.fel.pr1.data;

/**
 * The TimeSpan class represents a time interval
 * @author balikm1
 */
public final class TimeSpan {
    private int hour;
    private int minute;
    private int second;

    //<editor-fold defaultstate="collapsed" desc="initialization">
    /**
     * Constructs a newly allocated TimeSpan object initialized to zero
     */
    public TimeSpan() {
        this(0, 0, 0);
    }

    /**
     * Constructs a newly allocated TimeSpan object initialized to a specified number of seconds
     * @param second number of seconds
     */
    public TimeSpan(int second) {
        this(second / 60 / 60, (second / 60) % 60, second % 60);
    }

    /**
     * Constructs a newly allocated TimeSpan object initialized to a specified number of minutes, and seconds
     * @param minute number of minutes
     * @param second number of seconds
     */
    public TimeSpan(int minute, int second) {
        this(minute / 60, minute % 60, second);
    }

    /**
     * Constructs a newly allocated TimeSpan object initialized to a specified number of hours, minutes, and seconds
     * @param hour number of hours
     * @param minute number of minutes
     * @param second number of seconds
     */
    public TimeSpan(int hour, int minute, int second) {
        // calls the setTime method to validate the time
        setTime(hour, minute, second);
    }
    
    /**
     * Copy constructor.
     * Constructs a newly allocated TimeSpan object initialized to a time interval specified by an existing TimeSpan object
     * @param time the time interval used to initialize the new object
     */
    public TimeSpan(TimeSpan time) {
        this(time.getHour(), time.getMinute(), time.getSecond());
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="properties">
    /**
     * Set the hours component of the time interval
     * @param value the hours component of the time interval
     */
    public void setHour(int value) {
        hour = ((value >= 0 && value < 24) ? value : 0);
    }

    /**
     * Set the minutes component of the time interval
     * @param value the minutes component of the time interval
     */
    public void setMinute(int value) {
        minute = ((value >= 0 && value < 60) ? value : 0);
    }

    /**
     * Set the seconds component of the time interval
     * @param value the seconds component of the time interval
     */
    public void setSecond(int value) {
        second = ((value >= 0 && value < 60) ? value : 0);
    }

    /**
     * Get the hours component of the time interval
     * @return the hours component of the time interval
     */
    public int getHour() {
        return hour;
    }

    /**
     * Get the minutes component of the time interval
     * @return the minutes component of the time interval
     */
    public int getMinute() {
        return minute;
    }

    /**
     * Get the seconds component of the time interval
     * @return the seconds component of the time interval
     */
    public int getSecond() {
        return second;
    }
    //</editor-fold>
    
    //<editor-fold defaultstate="collapsed" desc="public methods">
    /**
     * Initializes the time interval to a specified number of hours, minutes, and seconds
     * @param hour number of hours
     * @param minute number of minutes
     * @param second number of seconds
     */
    public void setTime(int hour, int minute, int second) {
        setHour(hour);
        setMinute(minute);
        setSecond(second);
    }
    
    /**
     * Adds the specified number of seconds to the current intstance of the TimeSpan
     * @param second number of seconds to add
     * @return a reference to the modified TimeSpan instance
     */
    public TimeSpan add(int second) {
        this.second = (this.second + second)  % 60;
        this.minute = this.minute + ((second / 60) % 60);
        this.hour = this.hour + (second / 60 / 60);
        return this;
    }
    
    /**
     * Get the total number of seconds of the time interval
     * @return the total number of seconds of the time interval
     */
    public int getTotalSeconds() {
        return hour*60*60 + minute*60 + second;
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="object overrides">
    @Override
    public int hashCode() {
        int hash = 3;
        hash = 19 * hash + this.hour;
        hash = 19 * hash + this.minute;
        hash = 19 * hash + this.second;
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final TimeSpan other = (TimeSpan) obj;
        if (this.hour != other.hour) {
            return false;
        }
        if (this.minute != other.minute) {
            return false;
        }
        if (this.second != other.second) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return String.format("%d hours %d minutes %s seconds", hour, minute, second);
    }
    //</editor-fold>
}
